// Status bar drag and drop functionality based off of example code from
// http://kb.mozillazine.org/Dev_:_Extensions_:_Example_Code_:_Adding_Drag_and_Drop_to_Statusbarpanel
//

const gYm_flavour = "ym_notifier/statusbarpanel";        
const gYm_statusbarPanel = "ym-notifier-statusbar-item"; 
const gYm_statusbar = "status-bar";

const ym_statObserver = {
	
  dragging : false,
  	
  init: function () {
    // Removing the event listener in older browsers causes the extension not to work do to a bug
    // that was fixed in Firefox 1.5.  So don't do it!
    try {
      if (document.getElementById("content").mCurrentTab.linkedBrowser) oldBrowser = false;
    } catch (ex) {}
    if (!oldBrowser) {
      window.removeEventListener('load', ym_statObserver.init, false); 
    }
  
	// If Organize Status Bar installed, just exit and let OSB handle positioning the icon
	if (typeof(objOSB) != "undefined") return;
  
    var statusbarPanel = document.getElementById(gYm_statusbarPanel);
    var insertbefore = statusbarPanel.getAttribute("insertbefore");
    var insertafter = statusbarPanel.getAttribute("insertafter");

    // clear last element indicator attached to.
    ym_statObserver.lastElm = null;

    // if insertbefore or insertafter attributes are set obey the settings and
    // place our statusbar panel before or after the specified panel
    if (insertbefore) {
      document.getElementById(gYm_statusbar).
               insertBefore(statusbarPanel,
                            document.getElementById(statusbarPanel.getAttribute("insertbefore")));
    }
    else if (insertafter) {
      var prev = document.getElementById(statusbarPanel.getAttribute("insertafter"));
      if (prev) prev = prev.nextSibling;
      if (prev) 
        document.getElementById(gYm_statusbar).insertBefore(statusbarPanel, prev);
      else
        document.getElementById(gYm_statusbar).appendChild(statusbarPanel);
    }
  },

  getSupportedFlavours: function () {
    var flavours = new FlavourSet();
    flavours.appendFlavour(gYm_flavour); 
    return flavours;
  },

  // when user begins to drag, make sure every statusbarpanel has an id and
  // set up the event handlers for the drag and drop
  //
  onDragStart: function (evt,transferData,action) {
    var elme = evt.target;
    while(elme.id != gYm_statusbarPanel) {
      elme =elme.parentNode;
    }
    var txt=elme.getAttribute("id");
    transferData.data=new TransferData();
    transferData.data.addDataForFlavour(gYm_flavour,txt);
    ym_statObserver.dragSetup();
  },

  
  // clean up when release mouse without dropping on statusbar
  // - This fires a lot more often in Firefox 3.0 so now we check to make sure we aren't over the statusbar
  //   before cleaning up and if we cleaned up and are still dragging over the bar, setup again.
  //
  onDragExit: function(evt, session) { 
	var elm = evt.target;
	while (elm.parentNode && elm.id != gYm_statusbar) elm = elm.parentNode;
    if (!elm.id && session.sourceNode == document.getElementById(gYm_statusbarPanel)) ym_statObserver.scheduleCleanup();
    else if (elm.id && !elm.getAttribute("ymdrag")) {
	  clearTimeout(window.ymDragCleanup);
    }
  },

  // highlite the spot where the drop will occur
  //
  onDragOver: function (evt,flavour,session) {
    var rightleft = "";
    var elm = evt.target;
    while(elm.parentNode.nodeName.toLowerCase() != "statusbar") {
      elm = elm.parentNode;
    }

    // this will display a little line where the drop will occur
    var midPointCoord = elm.boxObject.x + (elm.boxObject.width/2);      // the offset + midpoint
    midPointCoord += !elm.previousSibling ? (elm.boxObject.width/4) :   // + additional quarter-coverage for left-most panel
                     (!elm.nextSibling ? (-elm.boxObject.width/4) : 0); // - less quarter-coverage for right-most panel
    if (evt.clientX < midPointCoord)
      rightleft = "left";
    else
      rightleft = "right";
    
    // remove old indicator if not the same
    if ((ym_statObserver.lastElm != null) && (ym_statObserver.lastElm != elm) ||
        ((ym_statObserver.lastElm == elm) && (ym_statObserver.lastElm.getAttribute("ymdrag") != rightleft)))
      ym_statObserver.lastElm.removeAttribute("ymdrag");

    elm.setAttribute("ymdrag", rightleft);
    ym_statObserver.lastElm = elm;
	
    evt.preventDefault();
  },

  // move the icon to the dropped location and save the location for future sessions
  //
  onDrop: function (evt,dropdata,session) {
    if (dropdata.data!="" && ym_statObserver.dragging) {

      var droppedPanel = document.getElementById(dropdata.data);
      var parent = droppedPanel.parentNode;
      var prev = evt.target;
      while (prev.nodeName.toLowerCase() != "statusbarpanel") {
        prev = prev.parentNode;
      }

      // this allows user to drop before or after the object under the mouse pointer depending on 
      // where the actual drop occurs.
      var midPointCoord = prev.boxObject.x + (prev.boxObject.width/2); // the offset + midpoint
      midPointCoord += !prev.previousSibling ? (prev.boxObject.width/4) : // + additional quarter-coverage for left-most panel
                       (!prev.nextSibling ? (-prev.boxObject.width/4) : 0); // - less quarter-coverage for right-most panel
      if (evt.clientX < midPointCoord)
        prev = prev.previousSibling;
      var next = (!prev) ? parent.firstChild : (prev.nextSibling != droppedPanel) ? 
                                                prev.nextSibling : prev.nextSibling.nextSibling;
		
      // store the insertbefore or insertafter attribute.  Our overlay is set up so 
      // that both these attributes will persist between browser sessions.
      if (prev) {
        droppedPanel.removeAttribute("insertbefore");
        droppedPanel.setAttribute("insertafter",""+prev.getAttribute("id"));
      }
      else if (next) {
        droppedPanel.removeAttribute("insertafter");
        droppedPanel.setAttribute("insertbefore",""+next.getAttribute("id"));
      }

      // do the actual move
      parent.removeChild(droppedPanel);
      if (next) 
        parent.insertBefore(droppedPanel,next);
      else
        parent.appendChild(droppedPanel);

      ym_statObserver.scheduleCleanup();
      ym_statObserver.dragging = false;
    }
  },

  // drag setup
  dragSetup: function() {
    var status = document.getElementById(gYm_statusbarPanel);
    var statusbar = document.getElementById(gYm_statusbar);
    statusbar.setAttribute("ymdrag", true);
    var child = statusbar.firstChild;
    var x = 0;
    while (child) {
      if (child != status) {
        // make sure every panel has an id and make that id persistent.  For some reason
        // the persist attribute isn't sticking.  I'm not sure why but this could cause problems.
        if (!child.id) {
          var newId = "statusbarpanel-noID"+x;
          while (document.getElementById(newId)) newId += "x"+x;
          child.id = newId;
          child.setAttribute("persist", new String("id" + (child.persist ? " "+child.persist : "")) );
          x++;
        }
        child.addEventListener("dragover", function(event) { nsDragAndDrop.dragOver(event,ym_statObserver); }, false);
        child.addEventListener("dragdrop", function(event) { nsDragAndDrop.drop(event,ym_statObserver); }, false);
        child.addEventListener("drop", function(event) { nsDragAndDrop.drop(event,ym_statObserver); }, false);
      }
      child = child.nextSibling;
    }
    // this will restore status bar to previous state if user does not drop on status bar
    window.addEventListener("dragexit", function(event) { nsDragAndDrop.dragExit(event,ym_statObserver); }, true);
    this.dragging = true;
  },
  
  // calling cleanUp directly fires immediately so use this to get around that
  //
  scheduleCleanup: function() {
    clearTimeout(window.ymDragCleanup);
    window.ymDragCleanup = setTimeout(ym_statObserver.cleanUp,250);
  },

  // restore the statusbar to its normal state
  //
  cleanUp: function() {
    clearTimeout(window.ymDragCleanup);

    var statusbar = document.getElementById(gYm_statusbar);
    var statusbarPanel = document.getElementById(gYm_statusbarPanel);
    statusbar.removeAttribute("ymdrag");

    // clear last element indicator attached to.
    ym_statObserver.lastElm = null;

    // remove the event handlers
    var child = statusbar.firstChild;
    var x = 0;
    while (child) {
      if (child != statusbarPanel) {
        child.removeAttribute("ymdrag");
        child.removeEventListener("dragover", function(event) { nsDragAndDrop.dragOver(event,ym_statObserver); }, false);
        child.removeEventListener("dragdrop", function(event) { nsDragAndDrop.drop(event,ym_statObserver); }, false);
      }
      child = child.nextSibling;
    }
    window.removeEventListener("dragexit", function(event) { nsDragAndDrop.dragExit(event,ym_statObserver); }, true);
    ym_statObserver.dragging = false;
  }
};

// its go time
window.addEventListener('load', ym_statObserver.init, false); 